/*  -----------------------------------------------------------------------
    tish - unixlike shell & tools for the TI-92+ and Voyage 200 calculators
    Copyright (C) 2020 Adam Colley <mog@kupo.be>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
------------------------------------------------------------------------ */

// cp command
// this is part of tish

#define USE_TI92PLUS
#define USE_V200
 
#include <tigcclib.h>
  
void _main(void)
{
  ESI argptr = top_estack;
  int args;
  SYM_ENTRY *SymPtr;
  char tname[50];
  unsigned char* sbuf;
  unsigned char* dbuf;
  unsigned short len;
  HANDLE sh;
  HANDLE dh;
  HANDLE h;
  HSym hsym;
 
  args=RemainingArgCnt(argptr);

  if (args!=2)
  {
    printf("Incorrect number of parameters\n");
    return;
  }  

  char *source = GetStrnArg (argptr);
  top_estack = next_expression_index (top_estack);
  char *destination = GetStrnArg (argptr);
  top_estack = next_expression_index (top_estack);

  SymPtr=DerefSym(SymFind(SYMSTR(source)));
  if (!SymPtr)
  {
    printf("Source file not found\n");
    return;
  }

  if (SymPtr->flags.bits.twin)
  {
    printf("File is twin, cannot modify\n");
    return;
  }
 
  SymPtr=DerefSym(SymFind(SYMSTR(destination)));
  if (SymPtr)
  {
    printf("Destination file already exists\n");
    return;
  }

  // basic checks complete, begin copy operation
  
  SymPtr=DerefSym(SymFind(SYMSTR(source)));
  sh=SymPtr->handle;
  sbuf=HLock(sh);
  len=*(unsigned short*)(sbuf)+2;
  
  if (!(dh=HeapAlloc(len)))
  {
    printf("Insufficient RAM for copy\n");
    HeapUnlock(sh);
    return;
  }
  
  dbuf=HLock(dh);
  memcpy(dbuf,sbuf,len);
  HeapUnlock(sh);
  HeapUnlock(dh);
  
  hsym=SymAdd(SYMSTR(destination));
  if (!hsym.folder || !(SymPtr=DerefSym(hsym)))
  {
    printf("Unable to create file\n");
    HeapFree(dh);
    return;
  }
  SymPtr->handle=dh;       
}
