/*  -----------------------------------------------------------------------
    tish - unixlike shell & tools for the TI-92+ and Voyage 200 calculators
    Copyright (C) 2020 Adam Colley <mog@kupo.be>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
------------------------------------------------------------------------ */

 // ls command
 // this is part of tish

 #define USE_TI92PLUS
 #define USE_V200
 
 #include <tigcclib.h>
 
 void pdir(char *,int);
 int gottts(void);
 
 void _main(void)
 {
   ESI argptr = top_estack;
   char pwd[64];
   int args,cnt;
   int dfmt=1;
 
   args=RemainingArgCnt(argptr);
   cnt=args;

   FolderGetCur(pwd);
 
   while (cnt)
   {
     char *arg = GetStrnArg (argptr);
     cnt--;
     top_estack = next_expression_index (top_estack);
     if (strstr(arg,"-h"))
       dfmt=0;
     if (strstr(arg,"-l"))
       dfmt=2;
  if ((!strchr(arg,'-')) && (strlen(arg)<9))
       sprintf(pwd,"%s",arg);
   }

   if (dfmt==0)
     sprintf(pwd,"\x7F");     

   //   printf("ls dfmt: [%d]\nls pwd : [%s]\n",dfmt,pwd); 
   pdir(pwd,dfmt);
 }
 
 void pdir(char *pwd, int dfmt)
 {
   SYM_ENTRY *SymPtr;
   HANDLE h;
   unsigned short count=0;
   unsigned short plen;
   char *cptr;
   char sbuf[40];
   char ftype[5]="----";
   unsigned char ftyp;
   char isexec=0;
   int tul;
 
   FolderOp(NULL,FOP_ALL_FOLDERS | FOP_LOCK);
   if (dfmt!=0)
     SymPtr=SymFindFirst(SYMSTR(pwd),FO_SINGLE_FOLDER);
   else  
     SymPtr=SymFindFirst(NULL,0);
   while (SymPtr)
   {
     count++;
     h=SymPtr->handle;
     cptr=HeapDeref(h);
     plen=*(short*)(cptr)+3;
     ftyp=cptr[plen-2];
     switch (ftyp)
     {
       case 0xf3:
         isexec=1;
         sprintf(ftype,"asm ");
         if (!strncmp(&cptr[6],"68kP",4))
         {
           sprintf(ftype,"kasm");
           if (!gottts())
             isexec=0;
         }
         if (!strncmp(&cptr[6],"68kL",4))
         {
           sprintf(ftype,"klib");
           isexec=0;
 
         }
         break;
        
       case 0xf8:
         isexec=0;
         strncpy(ftype,&cptr[plen-7],4);
         if (!ftype[0])
           strncpy(ftype,&cptr[plen-6],4);
         if ((!strncmp(ftype,"ppg",3)) && (gottts()))
           isexec=1;
         break;
        
       case 0xe0:
         isexec=0;
         sprintf(ftype,"text");  
         if (!strncmp(&cptr[5],"#tish",5))
         {
           sprintf(ftype,"tish");
           isexec=1;
         }
         break;
          
       case 0xdc:
         isexec=0;
         sprintf(ftype,"basp");  
         // check also for basf
         break;
          
       case 0x1f:
         isexec=0;
         sprintf(ftype,"vint");  
         break;
          
       case 0x23:
         isexec=0;
         sprintf(ftype,"vfp");  
         break;
          
       case 0x2d:
         isexec=0;
         sprintf(ftype,"vstr");  
         break;
          
       case 0xd9:
         isexec=0;
         sprintf(ftype,"vmat");  
         break;
          
       case 0xdd:
         isexec=0;
         sprintf(ftype,"vdat");  
         break;
          
       default:
         isexec=0;
         sprintf(ftype,"[%02x]",ftyp);
     }
     
   for (tul=0; tul<4; tul++)
     ftype[tul]=tolower(ftype[tul]);
   
     if (dfmt<2)
     {
       printf(" %-8s ",SymPtr->name);
       if (SymPtr->flags.bits.folder)
         printf("<F>");
       else
         printf("%2dK",(plen+1023)/1024);
       if (count%3==0)
         printf("\n");
     }
     else
     {
       sprintf(sbuf," %-8s  %5u  %-4s  -----  0x%06lx\n",SymPtr->name,plen,ftype,cptr);
       if (SymPtr->flags.bits.archived)
         sbuf[24]='a';
       if (SymPtr->flags.bits.locked)
         sbuf[25]='l';
       if (SymPtr->flags.bits.hidden)
         sbuf[26]='h';
       if (SymPtr->flags.bits.twin)
         sbuf[27]='t';
       if (isexec)
         sbuf[28]='x';
       printf(sbuf);
     }
     SymPtr=SymFindNext();
   }
   FolderOp(NULL,FOP_ALL_FOLDERS | FOP_UNLOCK);
   if ((dfmt<2) && (count%3!=0))
     printf("\n");
 }
 
 int gottts(void)
 {  
   SYM_ENTRY *SymPtr;
  
   SymPtr = DerefSym (SymFind (SYMSTR ("tish\\ttstart")));
   if (!SymPtr)
     return FALSE;
   return TRUE;
 }   
